Package: pcaMethods
Maintainer: Henning Redestig <henning.red@gmail.com>
License: GPL (>= 3)
Title: A collection of PCA methods
LinkingTo: Rcpp
LazyLoad: Yes
Author: Wolfram Stacklies, Henning Redestig, Kevin Wright
Authors@R: 
  c(person(given="Wolfram",
           family="Stacklies",
           role=c("aut")),
    person(given="Henning",
           family="Redestig",
           email="henning.red@gmail.com",
           comment=c(ORCID="0000-0003-2130-9288"),
           role=c("aut","cre")),
    person(given="Kevin",
           family="Wright",
           comment=c(ORCID="0000-0002-0617-8673"),
           role="aut"))
SystemRequirements: Rcpp
Description: Provides Bayesian PCA, Probabilistic PCA, Nipals PCA,
    Inverse Non-Linear PCA and the conventional SVD PCA. A cluster
    based method for missing value estimation is included for
    comparison. BPCA, PPCA and NipalsPCA may be used to perform PCA
    on incomplete data as well as for accurate missing value
    estimation. A set of methods for printing and plotting the
    results is also provided. All PCA methods make use of the same
    data structure (pcaRes) to provide a common interface to the
    PCA results. Initiated at the Max-Planck Institute for
    Molecular Plant Physiology, Golm, Germany.
Version: 2.4.0
URL: https://github.com/hredestig/pcamethods
BugReports: https://github.com/hredestig/pcamethods/issues
Encoding: UTF-8
Depends: Biobase, methods
Imports: BiocGenerics, Rcpp (>= 0.11.3), MASS
Suggests: matrixStats, lattice, ggplot2
Collate: 'derrorHierarchic.R' 'errorHierarchic.R' 'AllClasses.R'
        'AllGenerics.R' 'BPCA_dostep.R' 'BPCA_initmodel.R' 'bpca.R'
        'checkData.R' 'forkNlpcaNet.R' 'kEstimate.R' 'kEstimateFast.R'
        'lineSearch.R' 'llsImpute.R' 'methods-ExpressionSet.R'
        'methods-nniRes.R' 'methods-pcaRes.R' 'nipalsPca.R' 'nlpca.R'
        'optiAlgCgd.R' 'orth.R' 'pca.R' 'pcaMethods-package.R' 'ppca.R'
        'prep.R' 'repmat.R' 'robustPca.R' 'sortFeatures.R'
        'svdImpute.R' 'vector2matrices.R' 'xval.R'
Packaged: 2026-04-28 23:50:20 UTC; biocbuild
biocViews: Bayesian
RoxygenNote: 6.1.1
git_url: https://git.bioconductor.org/packages/pcaMethods
git_branch: RELEASE_3_23
git_last_commit: 4b5f83b
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
