# Generated by roxygen2: do not edit by hand

S3method(biplot,pcaRes)
S3method(dim,pcaRes)
S3method(fitted,pcaRes)
S3method(loadings,pcaRes)
S3method(plot,pcaRes)
S3method(predict,pcaRes)
S3method(residuals,pcaRes)
S3method(scores,pcaRes)
S3method(summary,pcaRes)
export(Q2)
export(RnipalsPca)
export(asExprSet)
export(bpca)
export(checkData)
export(cvseg)
export(kEstimate)
export(kEstimateFast)
export(listPcaMethods)
export(llsImpute)
export(nipalsPca)
export(nlpca)
export(nni)
export(pca)
export(plotPcs)
export(ppca)
export(prep)
export(robustPca)
export(robustSvd)
export(showNniRes)
export(showPcaRes)
export(svdImpute)
export(svdPca)
exportClasses(nlpcaNet)
exportClasses(nniRes)
exportClasses(pcaRes)
exportMethods(DModX)
exportMethods(R2VX)
exportMethods(R2cum)
exportMethods(center)
exportMethods(centered)
exportMethods(completeObs)
exportMethods(cvstat)
exportMethods(leverage)
exportMethods(loadings)
exportMethods(method)
exportMethods(nObs)
exportMethods(nP)
exportMethods(nPcs)
exportMethods(nVar)
exportMethods(nmissing)
exportMethods(sDev)
exportMethods(scaled)
exportMethods(scl)
exportMethods(scores)
exportMethods(slplot)
exportMethods(wasna)
import(Biobase)
import(BiocGenerics)
import(methods)
importFrom(Rcpp,evalCpp)
useDynLib(pcaMethods)
