% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-nniRes.R
\name{showNniRes}
\alias{showNniRes}
\title{Print a nniRes model}
\usage{
showNniRes(x, ...)
}
\arguments{
\item{x}{An \code{nniRes} object}

\item{...}{Not used}
}
\value{
Nothing, used for side-effect
}
\description{
Print a brief description of nniRes model
}
\author{
Henning Redestig
}
