% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vector2matrices.R
\docType{methods}
\name{vector2matrices,matrix-method}
\alias{vector2matrices,matrix-method}
\title{Tranform the vectors of weights to matrix structure}
\usage{
\S4method{vector2matrices}{matrix}(object, net)
}
\arguments{
\item{object}{an nlpcaNet}

\item{net}{the neural network}
}
\value{
weights in matrix structure
}
\description{
Tranform the vectors of weights to matrix structure
}
\author{
Henning Redestig
}
