% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{loadings,ANY-method}
\alias{loadings,ANY-method}
\alias{loadings}
\title{Crude way to unmask the function with the same name from
\code{stats}}
\usage{
\S4method{loadings}{ANY}(object, ...)
}
\arguments{
\item{object}{any object}

\item{...}{not used}
}
\value{
The loadings
}
\description{
Crude way to unmask the function with the same name from
\code{stats}
}
\author{
Henning Redestig
}
