% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{scaled,pcaRes-method}
\alias{scaled,pcaRes-method}
\alias{scaled}
\title{Check if scaling was part of the PCA model}
\usage{
scaled(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{Not used}
}
\value{
TRUE if scaling was part of the PCA model
}
\description{
Check if scaling was part of the PCA model
}
\author{
Henning Redestig
}
