% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{cvstat,pcaRes-method}
\alias{cvstat,pcaRes-method}
\alias{cvstat}
\title{Get cross-validation statistics (e.g. \eqn{Q^2}).}
\usage{
cvstat(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{not used}
}
\value{
vector CV statistics
}
\description{
Get cross-validation statistics (e.g. \eqn{Q^2}).
}
\author{
Henning Redestig
}
