% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{completeObs,nniRes-method}
\alias{completeObs,nniRes-method}
\alias{completeObs}
\alias{completeObs,pcaRes-method}
\title{Get the original data with missing values replaced with predicted
values.}
\usage{
completeObs(object, ...)
}
\arguments{
\item{object}{object to fetch complete data from}

\item{...}{Not used}
}
\value{
Completed data (matrix)
}
\description{
Get the original data with missing values replaced with predicted
values.
}
\author{
Henning Redestig
}
