% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-pcaRes.R
\docType{methods}
\name{nPcs,pcaRes-method}
\alias{nPcs,pcaRes-method}
\alias{nPcs}
\title{Get number of PCs.}
\usage{
nPcs(object, ...)
}
\arguments{
\item{object}{pcaRes object}

\item{...}{not used}
}
\value{
Number of PCs
}
\description{
Get number of PCs.
}
\note{
Try to use \code{link{nP}} instead since \code{nPcs} tend to
clash with argument names.
}
\author{
Henning Redestig
}
