% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lineSearch.R
\name{linr}
\alias{linr}
\title{Linear kernel}
\usage{
linr(x)
}
\arguments{
\item{x}{datum}
}
\value{
Input value
}
\description{
Linear kernel
}
\author{
Henning Redestig, Matthias Scholz
}
